# ---
# jupyter:
#   jupytext:
#     cell_metadata_filter: -all
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.14.4
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# +
# !pip install google-auth google-auth-oauthlib google-auth-httplib2 google-api-python-client

# First, install the required libraries
# !pip install google-auth google-auth-oauthlib google-auth-httplib2 google-api-python-client

# Import the libraries
from google.oauth2.credentials import Credentials
from googleapiclient.discovery import build


import os
import time
import io
import base64
import json
import google.auth
import googleapiclient.discovery
from googleapiclient.http import MediaFileUpload
import unittest
import logging

from googleapiclient.discovery import build
#from google.oauth2.service_account import ServiceAccountCredentials
from google.oauth2.service_account import Credentials
import json
# -




class Base_Extractor():
    cache_dir=""
    extract = ""
    extract_status_code = 0

    def __init__(self, cache_dir=""):
        self.cache_dir=cache_dir
        None


    #def extract(self, *argv):
    #    None

    def save_results_as_file(self, key_name):
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

        file_path = os.path.join(self.cache_dir, key_name)
        with open(file_path, "w") as file:
            if len(file_path)>3 and file_path[-4:] == "json":
                json.dump(self.extract, file)
            else:
                file.write(self.extract)

    def load_results_from_file(self, key_name):
        file_path = os.path.join(self.cache_dir, key_name)
        if os.path.isfile(file_path):
            with open(file_path, "r") as file:
                if len(file_path)>3 and file_path[-4:] == "json":
                    self.extract = json.load(file)
                    self.extract_status_code = 1
                else:
                    self.extract = file.read()
                    self.extract_status_code = 1
                return True
        else:
            return False

    def set_query_details(self, *argv):
        None


# +
def measure_time(func):
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        print(f"Execution time: {end - start} seconds")
        return result
    return wrapper

@measure_time
def my_function():
    # Function implementation here
    time.sleep(2)

my_function()


# -

class Google_Sheets_Extractor(Base_Extractor):
    file_id = ""
    tab_name = ""
    data_range = ""
    credentials_json = ""

    def __init__(self, request_parameters, cache_dir=""):
        super().__init__(cache_dir=cache_dir)
        self.load_request_parameters(request_parameters)
        logging.debug("Request_Connector")

    def load_request_parameters(self, parameters):
        self.file_id = parameters['file_id'] if "file_id" in parameters else ""
        self.tab_name = parameters['tab_name'] if "tab_name" in parameters else ""
        self.data_range = parameters['data_range'] if "data_range" in parameters else []
        self.credentials_json = parameters['credentials_json'] if "credentials_json" in parameters else []

    def get_google_drive_service(self):
        creds = Credentials.from_service_account_info(self.credentials_json, scopes=["https://www.googleapis.com/auth/drive", "https://www.googleapis.com/auth/spreadsheets"])
        sheets_service = build("sheets", "v4", credentials=creds)
        return sheets_service

    @measure_time
    def extract_data(self, cache_key=""):
        logging.debug("0")
        if self.extract_status_code == 0:
            logging.debug("1")
            if self.load_results_from_file(cache_key):
                logging.debug("2")
                logging.debug(self.extract)
                return self.extract
            else:
                temp_status = self.extract_from_web()
                if cache_key != "":
                    self.save_results_as_file(cache_key)
                logging.debug("3 "+ str(temp_status))
        logging.debug(self.extract)
        logging.debug(type(self.extract))
        return self.extract

    def get_response_status_code(self):
        return self.extract_status_code

    def decode_credentials_json():
        credentials_string = base64.b64decode(os.environ['CREDENTIALS_JSON']).decode('ascii')
        credentials_json = json.loads(credentials_string)
        #pretty_json = json.dumps(credentials_json, indent=4)
        #print(pretty_json)
        return credentials_json

    def get_sheets_content(self):
        #Build the Google Sheets API client
        sheets_service = self.get_google_drive_service()
        #sheets_service = build("sheets", "v4", credentials=creds)

        # Define the range to read
        range_ = self.tab_name+"!"+self.data_range

        # Call the Sheets API to get the values
        result = sheets_service.spreadsheets().values().get(spreadsheetId=self.file_id, range=range_).execute()

        # Get the values from the API result
        values = result.get("values", [])

        # Print the values
        for row in values:
            print(row)
        self.extract = values
        return self.extract

    def extract_from_web(self):
        self.get_sheets_content()
        print(type(self.get_sheets_content()))
        return "200"

# +




def list_all_directories_files(service):
    # Example API call
    results = service.files().list(q="mimeType='application/vnd.google-apps.folder'", fields="nextPageToken, files(id, name)").execute()
    folders = results.get("files", [])

    # Print the name of the first folder
    if folders:
        for folder in folders:
            folder_id = folder['id']
            print(f"The first folder is named: {folder}")
            results = service.files().list(q=f"'{folder_id}' in parents", fields="nextPageToken, files(id, name)").execute()
            files = results.get("files", [])
            print(files)
            # Print the names of the files in the folder
            if files:
                print("The files in the folder are:")
                for file in files:
                    print(file)
            else:
                print("No files were found in the folder.")

    else:
        print("No folders were found.")


    folder_id = folders[0]['id'] #"FOLDER_ID"

    # Example API call to list files in the folder



from io import BytesIO

def save_file_in_folder(service, folder_id, file_name, file_content):
    mimeType = ""
    if file_name[-4:]=="json":
        mimeType = "application/json"
    else:
        mimeType ="text/plain"

    file_metadata = {
        "name": file_name,
        "parents": [folder_id],
        "mimeType": mimeType
    }

    with open(file_name, "w") as file:
        file.write(file_content)

    file_id = get_file_by_name(service, file_name, folder_id)

    if file_id:
        #file_metadata['id'] = file_id['id']
        media = MediaFileUpload(file_name, mimetype=mimeType)
        file = service.files().update(fileId=file_id['id'], media_body=media).execute()
    else:
        media = MediaFileUpload(file_name, mimetype=mimeType)
        file = service.files().create(body=file_metadata, media_body=media, fields="id").execute()

    print(f"File ID: {file['id']}")
    return file

def get_file_by_name(service, file_name, folder_id):
    query = "name='" + file_name + "' and trashed = false and parents in '" + folder_id + "'"
    results = service.files().list(q=query, fields="nextPageToken, files(id, name)").execute()
    items = results.get("files", [])
    if not items:
        return None
    return items[0]

#credentials_json = decode_credentials_json()
#print(json.dumps(credentials_json, indent=4))
#service = get_google_drive_service(credentials_json)
#list_all_directories_files(service)
#file_content = get_file_content("1GoXl2a3tsvJfvTqUYh3p4Dw2a1e7Pc0R")
#print(file_content)

#folder_id="1Ba27PI1No-5wkzEaop4zfKrcLZiB_-9z"
#save_file_in_folder(service, folder_id, "first_saved_file.txt", "Content is here and has been updated")

# -




# +
class Test_Google_Sheets_Extractor(unittest.TestCase):

    def test_Google_Sheets_Extractor(self):
        expected = [['Bamboo Test A1', 'Bamboo Test B1'],
                    ['Bamboo Test A2', 'Bamboo Test B2']]

        parameters = {
            "file_id": "12keD9VYi6yrQ4nP7JJh95M8lmyTqIJw-V4IocOcyjYM",
            "tab_name": "sheet1",
            "data_range": "A1:B2",
            "credentials_json": decode_credentials_json()
        }
        gdse = Google_Sheets_Extractor(parameters,"test_data")
        #sheets_content = gdse.extract_data("google_sheets_file.json")
        sheets_content = gdse.extract_data()
        self.assertEqual(sheets_content, expected, "Content should be read from file from Google Drive")


logging.basicConfig(level=logging.ERROR)

unittest.main(argv=[''], verbosity=2, exit=False)
# -
























logging.basicConfig(level=logging.ERROR)























# +

if __name__ == "__main__":
    unittest.main(argv=[''], verbosity=2, exit=False)
# -

# from transform.recon_dataset import Recon_DataSet
# from transform import data_translations



from connectors.googlesheets_connector import GoogleSheets_Connector
from connectors.jira_connector import Jira_Connector
from connectors.base_connector import Base_Connector

import pandas as pd
import json
import numpy as np

class Recon_DataSet():

    def __init__(self, connector = Base_Connector(), df_in = None):
        self.bc = connector # A Py Recon Tools connector, eg JIRA, GoogleSheets Connector
        self.df = df_in #dataframe that stores the data via Pandas dataframe
        self.transform_function = None

    def test_first(self):
        return "test_first"

    def extract_data(self):
        cells = self.bc.get_clean_data()
        self.df = pd.DataFrame.from_dict(cells)
        return self.df

    def transform_data(self):
        self.df = self.transform_function(self.df)
        return self.df

    def transform_data_with_function(self, function_in):
        self.df = function_in(self.df)
        return self.df



# ---
# jupyter:
#   jupytext:
#     cell_metadata_filter: -all
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.14.4
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# +
# !pip install google-auth google-auth-oauthlib google-auth-httplib2 google-api-python-client


import os
import time
import io
import base64
import json
import google.auth
import googleapiclient.discovery
from googleapiclient.http import MediaFileUpload
import unittest
import logging

from googleapiclient.discovery import build
#from google.oauth2.service_account import ServiceAccountCredentials
from google.oauth2.service_account import Credentials
import json
# -



class Base_Loader():
    cache_dir=""
    file_content = ""
    file_name = ""
    save_status_code = 0

    def __init__(self, cache_dir=""):
        self.cache_dir=cache_dir
        None


    #def extract(self, *argv):
    #    None

    def save_results_as_file(self, key_name):
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

        file_path = os.path.join(self.cache_dir, key_name)
        with open(file_path, "w") as file:
            file.write(self.extract)

    def load_results_from_file(self, key_name):
        file_path = os.path.join(self.cache_dir, key_name)
        if os.path.isfile(file_path):
            with open(file_path, "r") as file:
                self.extract = file.read()
                self.extract_status_code = 1
                return True
        else:
            return False

    def set_query_details(self, *argv):
        None


# +
def measure_time(func):
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        print(f"Execution time: {end - start} seconds")
        return result
    return wrapper

@measure_time
def my_function():
    # Function implementation here
    time.sleep(0.2)

my_function()


# -

class Google_Drive_File_Loader(Base_Loader):
    folder_id = ""
    mimeType = ""
    parent_id = ""
    credentials_json = ""
    

    def __init__(self, request_parameters, cache_dir=""):
        super().__init__(cache_dir=cache_dir)
        self.load_request_parameters(request_parameters)
        logging.debug("Request_Connector")

    def load_request_parameters(self, parameters):
        self.file_name = parameters['file_name'] if "file_name" in parameters else ""
        self.parent_id = parameters['parent_id'] if "parent_id" in parameters else ""
        self.mimeType = parameters['mimeType'] if "mimeType" in parameters else ""
        self.credentials_json = parameters['credentials_json'] if "credentials_json" in parameters else ""
        self.file_content = parameters['file_content'] if "file_content" in parameters else ""
        self.folder_id = parameters['folder_id'] if "folder_id" in parameters else ""
        
    def get_google_drive_service(self):
        credentials = Credentials.from_service_account_info(self.credentials_json, scopes=["https://www.googleapis.com/auth/drive"])
        # Build the Google Drive API client using the access token
        service = build("drive", "v3", credentials=credentials)
        return service         
        
    @measure_time
    def save_data(self, file_content):
        self.file_content = file_content
        self.save_file_in_folder()
        return self.file_content

    def get_response_status_code(self):
        return self.save_status_code

    
    def get_file_by_name(self, service, file_name, folder_id):
        query = "name='" + file_name + "' and trashed = false and parents in '" + folder_id + "'"
        results = service.files().list(q=query, fields="nextPageToken, files(id, name)").execute()
        items = results.get("files", [])
        if not items:
            return None
        return items[0]   
    
    def save_file_in_folder(self):
        mimeType = ""
        if self.file_name[-4:]=="json":
            mimeType = "application/json"
        else:
            mimeType ="text/plain"

        file_metadata = {
            "name": self.file_name,
            "parents": [self.folder_id],
            "mimeType": self.mimeType
        }

        with open(self.file_name, "w") as file:
            file.write(self.file_content)

        #if file already exists, then overwrite rathert than create a 2nd copy    
        file_id = self.get_file_by_name(self.get_google_drive_service(), self.file_name, self.folder_id)
        
        if file_id:
            #file_metadata['id'] = file_id['id']
            media = MediaFileUpload(self.file_name, mimetype=mimeType)
            file = self.get_google_drive_service().files().update(fileId=file_id['id'], media_body=media).execute()
        else:
            print ("creating file")
            media = MediaFileUpload(self.file_name, mimetype=mimeType)
            file = self.get_google_drive_service().files().create(body=file_metadata, media_body=media, fields="id").execute()    

        print ("returning file")        
        return file  
    
    def extract_from_web(self):
        self.get_file_content()
        return "200"



    def decode_credentials_json():
        credentials_string = base64.b64decode(os.environ['CREDENTIALS_JSON']).decode('ascii')
        credentials_json = json.loads(credentials_string)
        #pretty_json = json.dumps(credentials_json, indent=4)
        #print(pretty_json)
        return credentials_json   


# +
class Test_Google_Drive_File_Loader(unittest.TestCase):

    def test_Google_Drive_Loader(self):
        expected = "hello, first file saved!"
        
        parameters = {
            "folder_id": "1dM-9RcAtT3IRlV_McL7Dw-y1hT5it7Pl",
            "file_name": "Google_Drive_File_Loader_savefile.txt",
            "credentials_json": decode_credentials_json()
        }
        gdfe = Google_Drive_File_Loader(parameters,"../test_data")
        file_content = gdfe.save_data("hello, first file saved!")
        self.assertEqual(file_content, expected, "Content should be saved to Google Drive")


logging.basicConfig(level=logging.ERROR)

unittest.main(argv=[''], verbosity=2, exit=False)
# -


























# +



# Support function. Needs updating and testing

logging.basicConfig(level=logging.ERROR)

def list_all_directories_files(service):
    # Example API call
    results = service.files().list(q="mimeType='application/vnd.google-apps.folder'", fields="nextPageToken, files(id, name)").execute()
    folders = results.get("files", [])

    # Print the name of the first folder
    if folders:
        for folder in folders:
            folder_id = folder['id'] 
            print(f"The first folder is named: {folder}")
            results = service.files().list(q=f"'{folder_id}' in parents", fields="nextPageToken, files(id, name)").execute()
            files = results.get("files", [])
            print(files)
            # Print the names of the files in the folder
            if files:
                print("The files in the folder are:")
                for file in files:
                    print(file)
            else:
                print("No files were found in the folder.")

    else:
        print("No folders were found.")


    folder_id = folders[0]['id'] #"FOLDER_ID"


# -





















# +

if __name__ == "__main__":
    unittest.main(argv=[''], verbosity=2, exit=False)
# -

# from transform.recon_dataset import Recon_DataSet
# from transform import data_translations



import pandas as pd
import numpy as np
import json
from tabulate import tabulate
import datetime

def test_first():
    #print(pd.read_csv('sheetdf.csv', encoding='utf-8').to_numpy().tolist())

    return "test_first_data_translations"


def make_first_row_header(df):
    new_header = df.iloc[0] #Get the first row for the header
    df.columns = new_header
    df = df[1:] # remove the header row from the data
    return df

def drop_first_row(df):
    df = df.iloc[1: , :]
    return df

def drop_first_column(df):
    df = df.iloc[: , 1:]
    return df

def process_component_sheets_data(df):
    df = make_first_row_header(df)
    #df = df[1:] # remove first info line

    df = standardise_component_data(df)
    return df

def process_component_sheets_data2(df):
    df = df[2:] # remove first 2 info lines
    df = make_first_row_header(df)
    print(" DATA...")
    print(df)
    df = standardise_component_data(df)
    return df


def process_jira_components_data(df):
    df.columns = ["Time Stamp","id","name","owner","description"]
    df = standardise_component_data(df)
    return df

# remove empty string ids and convert index colum types to int
def standardise_component_data(df):
    #pd.to_numeric(df['id'], errors='coerce')
    #df = df.replace('NaN',0)
    #df = df.replace('',0)
    #df['id']=df['id'].fillna(0)
    #df['id']=df['id'].astype('int')
    #df.id = df.id.fillna(0)
    #print ("Just id=0")
    #print (df.id == 0)
    #df['First Season'] = (df['First Season'] > 1990).astype(int)
    df = df.set_index('id', drop=False)
    df.index.name = 'index'
    df.index = df.index.fillna(0)
    df.index = df.index.astype('int')
    return df

def print_debug(df, extra_info=""):
    print()
    print(extra_info)
    print(df.dtypes)
    print(df.to_string())
    print(df.to_numpy().tolist())
    print(tabulate(df, headers='keys', tablefmt='psql'))

def flatten_jira_components_with_datetime_stamp(component_json):
    return flatten_jira_components(component_json, datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

# Takes the standard JSON from eg  https://leadtechie.atlassian.net/rest/api/3/project/TEST/components
# and pulls this out to a flat format: time_stampe, id, name, owners, description
def flatten_jira_components(component_json, time_stamp='2022-06-27 22:54:45'):
    results = []
    for component in component_json:
        results.append( [ time_stamp,
            component["id"],
            component["name"],
            component['lead']['displayName'] if 'lead' in component else "<No Owner>",
            component['description'] if 'description' in component else "<No Description>"
            ])
    return results







# https://stackoverflow.com/questions/15891038/change-column-type-in-pandas
def coerce_df_columns_to_numeric(df, column_list):
    df[column_list] = df[column_list].apply(pd.to_numeric, errors='coerce')

def get_local_data():
    sheetdf = pd.read_csv('sheetdf.csv', encoding='utf-8')
    jiradf = pd.read_csv('jiradf.csv', encoding='utf-8')


    sheetdf['id']=sheetdf['id'].astype('int')
    jiradf['id']=jiradf['id'].astype('int')

    jiradf = jiradf.set_index('id', drop=False)
    sheetdf = sheetdf.set_index('id', drop=False)
    #jiradf.set_index('id', drop=False)
    #sheetdf.set_index('id', drop=False)

    #jiradf = jiradf.rename(columns={"id": "myid"})
    #sheetdf = sheetdf.rename(columns={"id": "myid"})
    jiradf.index.name = 'index'
    sheetdf.index.name = 'index'

    jiradf.index.name = 'index'
    sheetdf.index.name = 'index'

    #pd.to_numeric(s)

    jiradf.index = jiradf.index.astype('int')
    sheetdf.index = sheetdf.index.astype('int')

    sheetdf = sheetdf.drop(columns=['Time Stamp'])
    jiradf = jiradf.drop(columns=['Time Stamp'])

    #print("Meta Data")
    #print(sheetdf.columns)
    #print(jiradf.columns)
    #print(sheetdf.dtypes)
    #print(jiradf.dtypes)

    #print_datasets(sheetdf, jiradf)
    #print(sheetdf.columns)
    #print(jiradf.columns)

    print("changing column type")
    #jiradf.astype({'id': 'object'})
    #pd.to_numeric(jiradf["id"])
    #pd.to_numeric(sheetdf["id"])

    #print("sheetdf.info()")
    #print(sheetdf.info())

    #print("jiradf.info()")
    #print(jiradf.info())


def update_add_delete_data(new_data_df, existing_data_df):
    # Notes that didn't quite work...
    #https://moonbooks.org/Articles/How-to-replace-rows-of-a-dataframe-using-rows-of-another-dataframe-based-on-indexes-with-pandas-/
    #https://stackoverflow.com/questions/51394653/update-a-pandas-dataframe-with-data-from-another-dataframe
    #df = sheetdf.combine_first(new_data_df).reindex(jiradf.index)

    # Update values where the row is in old and new data sets
    #sheetdf.update(sheetdf[['id']].merge(jiradf, 'left'))

    #print_debug(existing_data_df, "pre result")

    new_data_ids =  new_data_df.loc[:,"id"].values
    existing_data_ids = existing_data_df.loc[:,"id"].values
    update_rows = np.intersect1d(new_data_ids, existing_data_ids)
    for i in update_rows:
        existing_data_df.loc[i] = new_data_df.loc[i]

    # add new lines to the end
    new_data_ids =  new_data_df.loc[:,"id"].values
    existing_data_ids = existing_data_df.loc[:,"id"].values
    new_rows = np.setdiff1d(new_data_ids, existing_data_ids)
    #print(new_data_ids, existing_data_ids, new_rows)
    #print_debug(existing_data_df, "pre result")

    for i in new_rows:
        existing_data_df.loc[i] = new_data_df.loc[i]

    new_rows = np.setdiff1d(new_data_ids, existing_data_ids)
    #print(new_data_ids, existing_data_ids, new_rows)
    #print_debug(existing_data_df, "pre result")


    #dataframe.at[index,'column-name']='new value'
    deleted_rows = np.setdiff1d(existing_data_ids, new_data_ids)
    #print(deleted_rows)
    for i in deleted_rows:
        existing_data_df.loc[i, 'name':] = ""

    #print(existing_data_df.dtypes)
    #print_debug(existing_data_df, "result")
    return existing_data_df



# https://stackoverflow.com/questions/24831543/base-64-encode-a-json-variable-in-python

import json
import base64
import gspread
import os

#Take the credentials.json file and base64 encode it so you can add as key in environment variable and GitHub secret

def decode_credentials_json():
    credentials_string = base64.b64decode(os.environ['CREDENTIALS_JSON']).decode('ascii')
    credentials_json = json.loads(credentials_string)
    #pretty_json = json.dumps(credentials_json, indent=4)
    #print(pretty_json)
    return credentials_json

def encode_json_file_to_base64(filename='../credentials.json'):
    with open(filename) as jsonfile:
        data = json.load(jsonfile)
        #print(type(data))  #dict
        datastr = json.dumps(data)
        #print(type(datastr)) #str
        #print(datastr)
        #encoded = base64.b64encode(datastr.encode('utf-8'))  #1 way
        encoded = base64.b64encode(datastr.encode('ascii'))  #1 way

    return encoded

def print_base64_data(filename='../credentials.json'):
    print()
    print('Base64 Encoded json credentials file')
    print(encode_json_file_to_base64(filename))
    print("COPY THE TEXT between the b' and ' above into the gihhub environemnt variable (CREDENTIALS_JSON)")
    print()
    print()


def decode_base64_to_string(base64in):
    return base64.b64decode(base64in).decode('ascii')

def set_env_variable(name, value):
    os.environ[name] = value #encoded.decode('ascii')

def get_env(name):
    return os.getenv(name)

if __name__ == '__main__':
    print_base64_data()



# ---
# jupyter:
#   jupytext:
#     cell_metadata_filter: -all
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.14.4
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# +
# !pip install google-auth google-auth-oauthlib google-auth-httplib2 google-api-python-client


import os
import time
import io
import base64
import json
import google.auth
import googleapiclient.discovery
from googleapiclient.http import MediaFileUpload
import unittest
import logging

from googleapiclient.discovery import build
#from google.oauth2.service_account import ServiceAccountCredentials
from google.oauth2.service_account import Credentials
import json
# -


def decode_credentials_json():
    credentials_string = base64.b64decode(os.environ['CREDENTIALS_JSON']).decode('ascii')
    credentials_json = json.loads(credentials_string)
    #pretty_json = json.dumps(credentials_json, indent=4)
    #print(pretty_json)
    return credentials_json

class Base_Loader():
    cache_dir=""
    file_content = ""
    file_name = ""
    save_status_code = 0

    def __init__(self, cache_dir=""):
        self.cache_dir=cache_dir
        None


    #def extract(self, *argv):
    #    None

    def save_results_as_file(self, key_name):
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

        file_path = os.path.join(self.cache_dir, key_name)
        with open(file_path, "w") as file:
            file.write(self.extract)

    def load_results_from_file(self, key_name):
        file_path = os.path.join(self.cache_dir, key_name)
        if os.path.isfile(file_path):
            with open(file_path, "r") as file:
                self.extract = file.read()
                self.extract_status_code = 1
                return True
        else:
            return False

    def set_query_details(self, *argv):
        None


# +
def measure_time(func):
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        print(f"Execution time: {end - start} seconds")
        return result
    return wrapper

@measure_time
def my_function():
    # Function implementation here
    time.sleep(0.2)

my_function()


# -

class Google_Sheets_Loader(Base_Loader):
    file_id = ""
    tab_name = ""
    data_range = ""
    credentials_json = ""

    def __init__(self, request_parameters, cache_dir=""):
        super().__init__(cache_dir=cache_dir)
        self.load_request_parameters(request_parameters)
        logging.debug("Request_Connector")

    def load_request_parameters(self, parameters):
        self.file_id = parameters['file_id'] if "file_id" in parameters else ""
        self.tab_name = parameters['tab_name'] if "tab_name" in parameters else ""
        self.data_range = parameters['data_range'] if "data_range" in parameters else []
        self.credentials_json = parameters['credentials_json'] if "credentials_json" in parameters else []

    def get_google_drive_service(self):
        creds = Credentials.from_service_account_info(self.credentials_json, scopes=["https://www.googleapis.com/auth/drive", "https://www.googleapis.com/auth/spreadsheets"])
        sheets_service = build("sheets", "v4", credentials=creds)
        return sheets_service

    @measure_time
    def extract_data(self, cache_key=""):
        logging.debug("0")
        if self.extract_status_code == 0:
            logging.debug("1")
            if self.load_results_from_file(cache_key):
                logging.debug("2")
                logging.debug(self.extract)
                return self.extract
            else:
                temp_status = self.extract_from_web()
                if cache_key != "":
                    self.save_results_as_file(cache_key)
                logging.debug("3 "+ str(temp_status))
        logging.debug(self.extract)
        logging.debug(type(self.extract))
        return self.extract

    def get_response_status_code(self):
        return self.extract_status_code


    def save_data(self, data_in):
        self.file_content = data_in
        #Build the Google Sheets API client
        sheets_service = self.get_google_drive_service()
        #sheets_service = build("sheets", "v4", credentials=creds)

        # Define the range to read
        range_ = self.tab_name+"!"+self.data_range

        body = {
            'range': range_,
            'values': self.file_content,
            'majorDimension': 'ROWS'
        }

        # Call the Sheets API to get the values
        result = sheets_service.spreadsheets().values().update(spreadsheetId=self.file_id, range=range_, body=body, valueInputOption='RAW').execute()
        print (result)
        return result

    def extract_from_web(self):
        self.save_data(self.file_content)
        print(type(self.get_sheets_content()))
        return "200"


# +
class Test_Google_Sheets_Loader(unittest.TestCase):

    def test_Google_Sheets_Loader(self):
        data_to_save = [['Bamboo Test E1', 'Bamboo Test F1'],
                        ['Bamboo Test E2', 'Bamboo Test F2']]

        parameters = {
            "file_id": "12keD9VYi6yrQ4nP7JJh95M8lmyTqIJw-V4IocOcyjYM",
            "tab_name": "Sheet1",
            "data_range": "E1:F2",
            "credentials_json": decode_credentials_json()
        }
        gdse = Google_Sheets_Loader(parameters,"../test_data")
        result = gdse.save_data(data_to_save)
        expected = ""
        self.assertEqual(result['updatedCells'], 4, "It should return that 4 cells were updated")


logging.basicConfig(level=logging.ERROR)

unittest.main(argv=[''], verbosity=2, exit=False)
# -


























# +



# Support function. Needs updating and testing

logging.basicConfig(level=logging.ERROR)

def list_all_directories_files(service):
    # Example API call
    results = service.files().list(q="mimeType='application/vnd.google-apps.folder'", fields="nextPageToken, files(id, name)").execute()
    folders = results.get("files", [])

    # Print the name of the first folder
    if folders:
        for folder in folders:
            folder_id = folder['id']
            print(f"The first folder is named: {folder}")
            results = service.files().list(q=f"'{folder_id}' in parents", fields="nextPageToken, files(id, name)").execute()
            files = results.get("files", [])
            print(files)
            # Print the names of the files in the folder
            if files:
                print("The files in the folder are:")
                for file in files:
                    print(file)
            else:
                print("No files were found in the folder.")

    else:
        print("No folders were found.")


    folder_id = folders[0]['id'] #"FOLDER_ID"


# -





















# +

if __name__ == "__main__":
    unittest.main(argv=[''], verbosity=2, exit=False)
# -

# from transform.recon_dataset import Recon_DataSet
# from transform import data_translations



from distutils.core import setup
setup(
  name = 'bamboo_connect',         # How you named your package folder (MyLib)
  packages = ['bamboo_connect','extractors','loaders','samples','support','transformers'],   # Chose the same as "name"
  version = '0.0.4',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Bamboo Connect is a lightweight ETL (Extract, Transform, Load) library with examples and templates. It enables developers to quickly extract, transform, reconcile and then load resulting data securely. This avoids time consuming manual error prone tasks.',   # Give a short description about your library
  author = 'Chris Rowe',                   # Type in your name
  author_email = 'leadtechie@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/LeadTechie/BambooConnect',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/LeadTechie/BambooConnect/archive/v_01.tar.gz',    # I explain this later on
  keywords = ['ETL', 'PANDAS'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
    'pandas',
    'google_spreadsheet',
    'google-auth-oauthlib',
    'gspread',
    'requests',
    'jira',
    'coverage',
    'ruamel-yaml',
    'tabulate'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3.8',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.9'
  ],
)



# ---
# jupyter:
#   jupytext:
#     cell_metadata_filter: -all
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.14.4
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---


# +
# #!pip install gspread
import gspread
import sys
import os
import pathlib


os.chdir("..")
try:
    import support.authentication_support as auth_sup
    import transformers.data_translations as dt
    from extractors.google_sheets_extractor import Google_Sheets_Extractor
    from loaders.google_sheets_loader import Google_Sheets_Loader
    from extractors.extractor import Request_Extractor
except Exception as e:
    print(e)    
finally:    
    os.chdir("./samples")



# -

import pandas as pd
import numpy as np
import json
import os
import unittest
import logging

# from transform.recon_dataset import Recon_DataSet
# from transform import data_translations

# +
def load_google_data():
    expected = [['TestComponent1'],
                ['TestComponent2'],
                ['TestComponent3']]

    parameters = {
        "file_id": "1CeClWOxaysZztz5V_wxfSvFmd8KzntLc3J67rkhukq4",
        "tab_name": "sheet1",
        "data_range": "A1:E",
        "credentials_json": auth_sup.decode_credentials_json()
    }
    gdse = Google_Sheets_Extractor(parameters,"../test_data/")
    sheets_content = gdse.extract_data()
    google_sheets_df = pd.DataFrame(sheets_content)
    google_sheets_df_updated = dt.process_component_sheets_data(google_sheets_df)
    print(google_sheets_df_updated)
    return google_sheets_df_updated

load_google_data()


# +
def generate_params_for_jira_component_list(email, token):
    request_params = {}
    request_params['headers'] = {}
    request_params['data'] = {}
    request_params['url'] = "https://leadtechie.atlassian.net/rest/api/3/project/TEST/components"
    request_params['post_type']="GET"
    request_params['auth'] = (email, token)
    request_params['headers'] =  {
            "Accept": "application/json",
            'Content-Type': 'application/json'
        }
    return request_params

def load_jira_data():
    jira_token = os.environ['JIRA_TOKEN']
    jira_email = os.environ['JIRA_EMAIL']
    request_params = generate_params_for_jira_component_list(jira_email, jira_token)

    re_components = Request_Extractor(request_params, cache_dir="./test_data/")
    components_string = re_components.extract_data("")
    components_json = json.loads(components_string)
    pretty_json = json.dumps(components_json, indent=4)
    flattened_result = dt.flatten_jira_components(components_json)
    pretty_json = json.dumps(flattened_result, indent=4)
    print (pretty_json)
    df = pd.DataFrame(flattened_result)
    df1 = dt.process_jira_components_data(df)
    return df1


# -



def print_both_datasets(rds1, rds2):
    print("JIRA")
    print(rds2)
    print()
    print("Google")
    print(rds1)
    print()

def do_the_reconciliation(rds1, rds2):
    print_both_datasets(rds1, rds2)
    result = dt.update_add_delete_data(rds2, rds1)
    #print("result...")
    #print(result)
    return result

# +
def read_clean_test_expected_results(tab):
    parameters = {
        "file_id": "1CeClWOxaysZztz5V_wxfSvFmd8KzntLc3J67rkhukq4",
        "tab_name": tab,
        "data_range": "B1:E",
        "credentials_json": auth_sup.decode_credentials_json()
    }
    gdse = Google_Sheets_Extractor(parameters,"../test_data/")
    sheets_content = gdse.extract_data()
    google_sheets_df = pd.DataFrame(sheets_content)
    print(google_sheets_df)
    return google_sheets_df


def read_clean_test_data():
    parameters = {
        "file_id": "1CeClWOxaysZztz5V_wxfSvFmd8KzntLc3J67rkhukq4",
        "tab_name": "SampleData",
        "data_range": "A2:E5",
        "credentials_json": auth_sup.decode_credentials_json()
    }
    gdse = Google_Sheets_Extractor(parameters,"../test_data/")
    sheets_content = gdse.extract_data()
    google_sheets_df = pd.DataFrame(sheets_content)
    print(google_sheets_df)
    return google_sheets_df

def write_clean_test_data(data_to_save):
    parameters = {
        "file_id": "1CeClWOxaysZztz5V_wxfSvFmd8KzntLc3J67rkhukq4",
        "tab_name": "Sheet1",
        "data_range": "A1:E4",
        "credentials_json": auth_sup.decode_credentials_json()
    }
    gdse = Google_Sheets_Loader(parameters,"../test_data")
    result = gdse.save_data(data_to_save)
    return result

def write_results_data(data_to_save):
    parameters = {
        "file_id": "1CeClWOxaysZztz5V_wxfSvFmd8KzntLc3J67rkhukq4",
        "tab_name": "Sheet1",
        "data_range": "A1:E",
        "credentials_json": auth_sup.decode_credentials_json()
    }
    gdse = Google_Sheets_Loader(parameters,"../test_data")
    result = gdse.save_data(data_to_save)
    return result


def load_jira_and_google_data_and_reconcile_and_save_results():
    clean_data = read_clean_test_data()
    write_clean_test_data(clean_data.values.tolist())

    rds1 = load_google_data()
    rds2 = load_jira_data()

    rds1[["id"]] = rds1[["id"]].astype(int)
    rds1.index = rds1.index.astype(int)

    rds2[["id"]] = rds2[["id"]].astype(int)
    rds2.index = rds2.index.astype(int)

    
    result = do_the_reconciliation(rds1, rds2)
    results = write_results_data(result.values.tolist())
    print(result)
    return result
    #write_results_data(result)
    
    #gsc.reset_sheet_data("Recon Tools Test Data", "write-data-test")
    #gsc.copy_sheet_data("Recon Tools Test Data", "SampleData", "write-data-test")
    #gsc.update_data("Recon Tools Test Data", "write-data-test", "A3", result.to_numpy().tolist() )
    #print("done")
    
  
# -



def new_e2e_test():
    load_google_data()
    load_jira_data()
    update_data()  


# +
class Test_Google_Sheets_Extractor(unittest.TestCase):

    def test_Google_Sheets_Extractor(self):
        load_jira_and_google_data_and_reconcile_and_save_results()
        actual = read_clean_test_expected_results("Sheet1") 
        expected = read_clean_test_expected_results("ExpectedResult") 

        self.assertEquals(actual.values.tolist(), expected.values.tolist(), "Check that after reconciliation the results match the expected")


logging.basicConfig(level=logging.ERROR)

unittest.main(argv=[''], verbosity=2, exit=False)
# -



if __name__ == '__main__':
    load_jira_and_google_data_and_reconcile_and_save_results()







# ---
# jupyter:
#   jupytext:
#     cell_metadata_filter: -all
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.14.4
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---



import time
import json
import os
import logging
import requests
import unittest









class Base_Extractor():
    cache_dir=""
    extract = ""
    extract_status_code = 0

    def __init__(self, cache_dir=""):
        self.cache_dir=cache_dir
        None


    #def extract(self, *argv):
    #    None

    def save_results_as_file(self, key_name):
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

        file_path = os.path.join(self.cache_dir, key_name)
        with open(file_path, "w") as file:
            file.write(self.extract)

    def load_results_from_file(self, key_name):
        file_path = os.path.join(self.cache_dir, key_name)
        if os.path.isfile(file_path):
            with open(file_path, "r") as file:
                self.extract = file.read()
                self.extract_status_code = 1
                return True
        else:
            return False

    def set_query_details(self, *argv):
        None



# +
def measure_time(func):
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        print(f"Execution time: {end - start} seconds")
        return result
    return wrapper

@measure_time
def my_function():
    # Function implementation here
    time.sleep(2)

my_function()
# -



class Request_Extractor(Base_Extractor):
    headers = {}
    data = {}
    url = "https://jsonplaceholder.typicode.com/todos/1"
    post_type="GET"
    auth = None

    response = None

    def __init__(self, request_parameters, cache_dir=""):
        super().__init__(cache_dir=cache_dir)
        self.load_request_parameters(request_parameters)
        logging.debug("Request_Connector")

    def load_request_parameters(self, request_parameters):
        self.headers = request_parameters['headers'] if "headers" in request_parameters else {}
        self.data = request_parameters['data'] if "data" in request_parameters else {}
        self.url = request_parameters['url'] if "url" in request_parameters else "https://jsonplaceholder.typicode.com/todos/1"
        self.post_type=request_parameters['post_type'] if "url" in request_parameters else "GET"
        self.auth=request_parameters['auth'] if "auth" in request_parameters else None

    @measure_time
    def extract_data(self, cache_key=""):
        logging.debug("0")
        if self.extract_status_code == 0:
            logging.debug("1")
            if self.load_results_from_file(cache_key):
                logging.debug("2")
                logging.debug(self.extract)
                return self.extract
            else:
                temp_status = self.extract_from_web()
                if cache_key != "":
                    self.save_results_as_file(cache_key)
                logging.debug("3 "+ str(temp_status))
        logging.debug(self.extract)
        logging.debug(type(self.extract))
        return self.extract

    def get_response_status_code(self):
        return self.extract_status_code

    def extract_from_web(self):
        response = requests.request(self.post_type, self.url, headers=self.headers, auth=self.auth, data=self.data)
        self.extract_status_code = response.status_code
        self.extract = response.content.decode()
        logging.debug("extract_from_web" + self.extract)
        return self.extract_status_code








def generate_params_for_jira_component_list(email, token):
    request_params = {}
    request_params['headers'] = {}
    request_params['data'] = {}
    request_params['url'] = "https://leadtechie.atlassian.net/rest/api/3/project/TEST/components"
    request_params['post_type']="GET"
    request_params['auth'] = (email, token)
    request_params['headers'] =  {
            "Accept": "application/json",
            'Content-Type': 'application/json'
        }
    return request_params





logging.basicConfig(level=logging.ERROR)


# +
class Test_Request_Extractor(unittest.TestCase):

    def test_placeholder_api_test(self):
        expected = {
          "userId": 1,
          "id": 1,
          "title": "delectus aut autem",
          "completed": False
        }

        rc = Request_Extractor({},"./test_data/")
        re = rc.extract_data("")
        re_json = json.loads(re)
        self.assertEqual(re_json, expected, "check json from placeholder external api")

        re = rc.extract_data("jsonplaceholder.json")
        re_json = json.loads(re)
        self.assertEqual(re_json, expected, "check json from external and cache result")

        re = rc.extract_data("jsonplaceholder.json")
        re_json = json.loads(re)
        self.assertEqual(re_json, expected, "check getting from in memory cache")

        Request_Extractor({},"./test_data/")
        re = rc.extract_data("jsonplaceholder.json")
        re_json = json.loads(re)
        self.assertEqual(re_json, expected, "check getting from in file cache")


logging.basicConfig(level=logging.ERROR)

unittest.main(argv=[''], verbosity=2, exit=False)
# -



# +


def test_repeated_calls_to_check_cache():
    jira_token = os.environ['JIRA_TOKEN']
    jira_email = os.environ['JIRA_EMAIL']
    request_params = generate_params_for_jira_component_list(jira_email, jira_token)

    re_components = Request_Extractor(request_params, cache_dir="./test_data/")
    components_string = re_components.extract_data("")
    components_json = json.loads(components_string)
    pretty_json = json.dumps(components_json, indent=4)
    #print(pretty_json)

    re_components = Request_Extractor(request_params, cache_dir="./test_data/")
    components_string = re_components.extract_data("")
    components_json = json.loads(components_string)
    pretty_json = json.dumps(components_json, indent=4)
    #print(pretty_json)

    re_components = Request_Extractor(request_params, cache_dir="./test_data/")
    components_string = re_components.extract_data("component_list.json")
    components_json = json.loads(components_string)
    pretty_json = json.dumps(components_json, indent=4)
    #print(pretty_json)

    re_components = Request_Extractor(request_params, cache_dir="./test_data/")
    components_string = re_components.extract_data("component_list.json")
    components_json = json.loads(components_string)
    pretty_json = json.dumps(components_json, indent=4)
    #print(pretty_json)

    components_string = re_components.extract_data("component_list.json")
    components_json = json.loads(components_string)
    pretty_json = json.dumps(components_json, indent=4)
    print(pretty_json)

test_repeated_calls_to_check_cache()
# -







# +

def flatten_jira_components_with_datetime_stamp(component_json):
    return flatten_jira_components(component_json, datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

# Takes the standard JSON from eg  https://leadtechie.atlassian.net/rest/api/3/project/TEST/components
# and pulls this out to a flat format: time_stampe, id, name, owners, description
def flatten_jira_components(component_json, time_stamp='2022-06-27 22:54:45'):
    results = []
    for component in component_json:
        results.append( [ time_stamp,
            component["id"],
            component["name"],
            component['lead']['displayName'] if 'lead' in component else "<No Owner>",
            component['description'] if 'description' in component else "<No Description>"
            ])
    return results
# -







# +
class Test_Extract_Component_List(unittest.TestCase):

    def test_component_list(self):
        expected = {
            "self": "https://leadtechie.atlassian.net/rest/api/3/component/10000",
            "id": "10000",
            "name": "TestComponent1",
            "description": "TestComponent1 Description",
            "assigneeType": "PROJECT_DEFAULT",
            "realAssigneeType": "PROJECT_DEFAULT",
            "isAssigneeTypeValid": False,
            "project": "TEST",
            "projectId": 10000
        }

        jira_token = os.environ['JIRA_TOKEN']
        jira_email = os.environ['JIRA_EMAIL']
        request_params = generate_params_for_jira_component_list(jira_email, jira_token)

        re_components = Request_Extractor(request_params, cache_dir="./test_data/")
        components_string = re_components.extract_data("")
        re_components_json = json.loads(components_string)

        pretty_json = json.dumps(re_components_json, indent=4)
        self.assertEqual(re_components_json[0], expected, "check first component")


logging.basicConfig(level=logging.ERROR)

unittest.main(argv=[''], verbosity=2, exit=False)

# +

if __name__ == "__main__":
    unittest.main(argv=[''], verbosity=2, exit=False)
# -

# from transform.recon_dataset import Recon_DataSet
# from transform import data_translations
